"""
Helper functions
"""

# Third Party
from dhooks_lite import Embed, Webhook
from discord import Embed as DiscordEmbed

# Django
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter
from allianceauth.framework.api.evecharacter import get_user_from_evecharacter
from allianceauth.services.hooks import get_extension_logger
from esi.models import Token

from .app_settings import AC_REQUIRED_SCOPES, AC_WEBHOOK, discord_bot_active

if discord_bot_active():
    # Third Party
    from aadiscordbot import tasks as bot_tasks  # pylint: disable=import-error


logger = get_extension_logger(__name__)


def member_tokens_count_for_corp(
    corp_id: int,
    quick_check: bool = False,
) -> tuple[int, list[str]]:
    """
    Returns the number of valid tokens for a given corp and a list of names without valid tokens.
    """
    members = EveCharacter.objects.filter(corporation_id=corp_id)
    invalid_names = []  # List to store names of members without valid tokens
    valid_count = 0  # Counter for valid tokens

    # Iterate over members and process each
    for member in members:
        if has_required_scopes_for_request(member, quick_check=quick_check):
            valid_count += 1  # Increment count if the token is valid
        else:
            invalid_names.append(
                member.character_name
            )  # Add name to the list if token is invalid
    return valid_count, invalid_names


def has_required_scopes_for_request(
    char: EveCharacter,
    quick_check: bool = False,
) -> bool:
    """Returns True if given character has the required scopes

    Params:
    - user: provide User object to shorten processing time
    - quick: if True will not check if tokens are valid to save time
    """
    # get the user and check they have the member state

    try:
        user = get_user_from_evecharacter(character=char)
        state_name = user.profile.state.name
    except ObjectDoesNotExist:
        return False

    if state_name in AC_REQUIRED_SCOPES:
        scopes = AC_REQUIRED_SCOPES[state_name]
        # the user has a valid state lets check the characters token
        scopes_string = " ".join(scopes)
        token_qs = Token.objects.filter(character_id=char.character_id).require_scopes(
            scopes_string
        )

    else:
        return False

    if not quick_check:
        token_qs = token_qs.require_valid()

    result = token_qs.exists()
    return result


def send_embed_message(
    webhook: bool = False, user: User = None, embed: Embed = None, cont: str = ""
):
    """Sends an embed message via webhook or user DM."""

    if webhook:
        hook = Webhook(
            AC_WEBHOOK,
            username="Alt Corp Alert",
        )
        hook.execute(embeds=[embed], content=cont)

    if user is not None:
        if discord_bot_active():
            e2 = DiscordEmbed.from_dict(embed.asdict())
            bot_tasks.send_message(user=user, embed=e2)



# Standard Library
from datetime import datetime

# Third Party
from dhooks_lite import Embed
from discord import Color

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.http import HttpResponse, HttpResponseNotFound
from django.shortcuts import render

# Alliance Auth
from allianceauth.framework.api.user import get_main_character_from_user
from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag

# Alt Corp
from altcorp import __title__
from altcorp.helpers import member_tokens_count_for_corp, send_embed_message
from altcorp.models import AltCorpRequest

from ._common import add_common_context

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required
@permission_required("altcorp.manage_requests")
def manage_requests(request):
    approved = AltCorpRequest.approved()
    danger = AltCorpRequest.danger()
    revoked = AltCorpRequest.revoked()
    context = {
        "approved": approved,
        "danger": danger,
        "revoked": revoked,
    }
    return render(
        request,
        "altcorp/manage_requests.html",
        add_common_context(request, context),
    )


def requests_list(request):
    """
    Get requests list
    """
    pending = AltCorpRequest.pending()
    corporations_data = []
    for corp in pending:
        tokens, unreg = member_tokens_count_for_corp(corp.corporation.corporation_id)
        main_character = get_main_character_from_user(user=corp.user)
        if corp.corporation.corporation_id == 98718236:
            tokens = corp.corporation.member_count
        row = {
            "ceo": main_character,
            "token_count": tokens,
            "unregistered": unreg,
            "request": corp,
        }
        corporations_data.append(row)
    corporations_data.sort(key=lambda x: (x["request"].corporation.corporation_name,))
    approved = AltCorpRequest.approved()
    corporations_data2 = []
    for corp in approved:
        tokens, unreg = member_tokens_count_for_corp(corp.corporation.corporation_id)
        main_character = get_main_character_from_user(user=corp.user)
        if corp.corporation.corporation_id == 98718236:
            tokens = corp.corporation.member_count
        row = {
            "ceo": main_character,
            "token_count": tokens,
            "unregistered": unreg,
            "request": corp,
        }
        corporations_data2.append(row)
    corporations_data2.sort(key=lambda x: (x["request"].corporation.corporation_name,))

    context = {"corps": corporations_data, "approved": corporations_data2}
    return render(
        request,
        "altcorp/partials/requests_list.html",
        context,
    )


def revocations_list(request):
    """
    Get revocations list
    """
    revoke = AltCorpRequest.expired_revoke_deadlines()
    corporations_data = []
    for corp in revoke:
        main_character = get_main_character_from_user(user=corp.user)
        row = {
            "ceo": main_character,
            "request": corp,
        }
        corporations_data.append(row)
    corporations_data.sort(key=lambda x: (x["request"].corporation.corporation_name,))
    danger = AltCorpRequest.danger()
    corporations_data2 = []
    for corp in danger:
        tokens, unreg = member_tokens_count_for_corp(corp.corporation.corporation_id)
        main_character = get_main_character_from_user(user=corp.user)
        row = {
            "ceo": main_character,
            "token_count": tokens,
            "unregistered": unreg,
            "request": corp,
        }
        corporations_data2.append(row)
    corporations_data2.sort(key=lambda x: (x["request"].corporation.corporation_name,))

    context = {"corps": corporations_data, "danger": corporations_data2}
    return render(
        request,
        "altcorp/partials/revocations_list.html",
        context,
    )


def manage_requests_write(request, request_id):
    """
    Approve/Reject a request
    """
    # approve request
    if request.method == "PUT":
        try:
            altcorp = AltCorpRequest.objects.get(id=request_id)
            altcorp.status = "APPROVED"
            altcorp.status_date = datetime.now()
            altcorp.save()
        except AltCorpRequest.DoesNotExist:
            return HttpResponseNotFound()

        desc = "Your alt corp application has been approved"
        e = Embed(
            title=f"{altcorp.corporation.corporation_name} Approved",
            description=desc,
            color=Color.green(),
        )
        send_embed_message(user=altcorp.user, embed=e)
        return HttpResponse("")
    # deny request
    if request.method == "DELETE":
        try:
            altcorp = AltCorpRequest.objects.get(id=request_id)
            desc = "Your alt corp application has been declined"
            e = Embed(
                title=f"{altcorp.corporation.corporation_name} Declined",
                description=desc,
                color=Color.red(),
            )
            send_embed_message(user=altcorp.user, embed=e)
            altcorp.delete()
        except AltCorpRequest.DoesNotExist:
            return HttpResponseNotFound()

        return HttpResponse("")

    return HttpResponseNotFound()



"""
common
"""

# Alt Corp
from altcorp.models import AltCorpRequest

DEFAULT_ICON_SIZE = 32


def add_common_context(request, context: dict) -> dict:
    """adds the common context used by all view"""
    pending = AltCorpRequest.pending().count()
    danger = AltCorpRequest.danger().count()
    revoke = AltCorpRequest.expired_revoke_deadlines().count()
    new_context = {
        **{
            "total_count": {
                "pending": pending,
                "danger": danger,
                "revoke": revoke,
            },
        },
        **context,
    }
    return new_context
